#!/bin/bash

shellfu import jat

#
# Persistent Relic Storage Tool
#

prst__append() {
    #
    # Append prst data value $2 to key $1
    #
    # See prst__path() for key syntax.
    #
    local key=$1    # key to write
    local value=$2  # value to write
    local tgt       # target file path
    tgt=$(prst__path "$key") || return $?
    echo "$value" >> "$tgt"
}

prst__cp() {
    #
    # Copy to key $1 items $2..
    #
    # See prst__path() for key syntax.
    #
    local key=$1    # key to write
    local tgt       # target file path
    tgt=$(prst__path "$key") || return $?
    mkdir -p "$tgt"
    cp -ar "$@" "$tgt"
}

prst__dir() {
    #
    # Create directory at key $1
    #
    # See prst__path() for key syntax.
    #
    local key=$1    # key to write
    local tgt       # target file path
    tgt=$(prst__path "$key") || return $?
    mkdir -p "$tgt"
}

prst__init() {
    #
    # Initialize storage at root $1
    #
    local nsaliases=()
    while true; do case $1 in
        -n|--ns-alias) nsaliases+=("$2" "$3"); shift 3 ;;
        -*) jat__log_error "unknown argument: $1"; return 2 ;;
        *)  break ;;
    esac done
    __PRST__ROOT=$1
    test -n "$__PRST__ROOT" || {
        jat__log_error "no root?"
        return 3
    }
    mkdir -p "$__PRST__ROOT/.prst"
    set -- "${nsaliases[@]}"
    while test $# -gt 0; do
        __prst__nsalias "$1" "$2"; shift 2
    done
}

prst__path() {
    #
    # Dereference storage key $1
    #
    # If the storage key name is prefixed by NS and period:
    #
    #     [NS.]KEY
    #
    # the NS must be previously defined using -n argument of
    # prst__init(), and the file path will contain
    # part generated by the alias code:
    #
    #     ROOT/NS_INFIX/KEY
    #
    # where ROOT is the main storage root, NS_INFIX is the part generated
    # by the alias code, and KEY is the key passed to this function (with
    # the NS. part removed).
    #
    # Note: This function has a side effect within the storage structure that
    # directory for the data file is automatically created so that caller does
    # not need to.
    #
    local key=$1    # key to dereference
    local adpath    # alias definition path
    local nsalias   # each alias found
    local path="$__PRST__ROOT/data/"
    local nsinfix   # storage path subdir under ^^
    local alist     # list of aliases string w/ newlines
    __prst__ckroot || return 2
    adpath="$__PRST__ROOT/.prst/aliases"
    alist="$(find "$adpath" -maxdepth 1 -mindepth 1 -printf '%P\n')"
    while read -r nsalias; do
        if grep -qe "^$nsalias\\." <<<"$key"; then
            if test -e "$adpath/$nsalias/code"; then
                nsinfix="$(eval "$adpath/$nsalias/code")" || {
                    jat__log_error \
                        "dealiasing code exited with non-zero status"
                    echo /dev/null
                    return 3
                }
                break
            fi
        fi
    done <<<"$alist"
    path+="$nsinfix/$key"
    mkdir -p "${path%/*}"
    echo "$path"
}

prst__read() {
    #
    # Get prst data from key $1
    #
    # See prst__path() for key syntax.
    #
    local key=$1    # key to read
    cat "$(prst__path "$key")" || return $?
}

prst__submit() {
    #
    # Submit tarball of storage root
    #
    local name
    __prst__ckroot || return 2
    name=$(tr -c '[:alnum:]_.+%@-' '-' <<<"$__PRST__ROOT")
    jat__submitb "$name.tar.gz" "$__PRST__ROOT"
}

prst__write() {
    #
    # Save prst data value $2 under key $1
    #
    # See prst__path() for key syntax.
    #
    local key=$1    # key to write
    local value=$2  # value to write
    local tgt       # target file path
    tgt=$(prst__path "$key") || return $?
    echo "$value" > "$tgt"
}


#          #                              your relics will be persistent #
# INTERNAL # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ #
#          #                          but useless if you cross this line #


#
# Storage root
#
__PRST__ROOT=${__PRST__ROOT:-}

__prst__nsalias() {
    #
    # Add alias of name $1 and code $2
    #
    local name=$1
    local code=$2
    local re_ok='[[:alnum:]_]+'
    local adpath
    grep -E "$re_ok" <<<"$name" || {
        jat__log_error \
            "invalid alias name: '$name' does not match '$re_ok'"
        return 2
    }
    adpath="$__PRST__ROOT/.prst/aliases/$name"
    mkdir -p "$adpath"
    bash -n <<<"$code" || {
        jat__log_error \
            "syntax error in alias code: '$code'"
        return 2
    }
    echo "$code" > "$adpath/code"
}

__prst__ckroot() {
    #
    # Check if storage root has been set and initialized
    #
    test -d "$__PRST__ROOT" || {
        jat__log_error \
            "storage root has not been initialized" \
            "did you run prst__init()?"
        return 3
    }
    return 0
}

#shellfu module-version=0.0.0
